#' compute Fisher-exact test https://en.wikipedia.org/wiki/Fisher%27s_exact_test
#' @description  compute fisher test pvalue and estimate in piped expression
#' @keywords internal
#' @export
compute_fisher <- function(a, b, c, d) {
  if (any(is.na(list(a, b, c, d)))) {
    stop("All inputs must be non-missing")
  }

  data <- matrix(c(a, b, c, d), ncol = 2)
  c(
    p_value = round(fisher.test(data, alternative = "greater")$p.value, 3),
    estimate = round(fisher.test(data, alternative = "greater")$estimate, 3)
  )
}

#' compute Barnard test https://en.wikipedia.org/wiki/Barnard%27s_test
#' @description  compute Barnard test pvalue and estimate in piped expression
#' @keywords internal
#' @export
compute_barnard <- function(a, b, c, d, method = "boschloo") {
  if (any(is.na(list(a, b, c, d)))) {
    stop("All inputs must be non-missing")
  }
  # only pre
  if ((a + b) > 0 & (c + d) == 0) {
    1
  }
  # only post
  else if ((a + b) == 0 & (c + d) > 0) {
    0
  } else {
    data <- matrix(c(d, b, c, a), ncol = 2)
    out_barn <- DescTools::BarnardTest(data, alternative = "greater", method = "boschloo") %>%
      suppressWarnings()
    c(
      p_value = round(out_barn$p.value, 5),
      estimate = round(out_barn$estimate, 3)
    )
  }
}

#' compute Z-test proportional
#' @description  compute Z-test pvalue and estimate in piped expression
#' @keywords internal
#' @export
compute_prop_test <- function(a, b, c, d, correct = FALSE) {
  if (any(is.na(list(a, b, c, d)))) {
    stop("All inputs must be non-missing")
  }

  m_1 <- a + b
  m_2 <- c + d
  p_1 <- b / m_1
  p_2 <- d / m_2
  diff_p2_p1 <- p_2 - p_1

  c(
    p_value = stats::prop.test(
      x = c(d, b),
      n = c(m_2, m_1),
      correct = correct,
      alternative = "greater"
    )$p.value %>% suppressWarnings(),
    estimate = stats::prop.test(
      x = c(d, b),
      n = c(m_2, m_1),
      correct = correct,
      alternative = "greater"
    )$estimate %>% suppressWarnings()
  )
}


#' Compute Winning rate across the crisis indicator
#'
#' @description
#' The indicator focuses on companies that after the emergency outbreak were awarded public contracts much more frequently than before the emergency.
#'
#' ### Motivation:
#' The red flag considers at risk companies that **exceptionally increase** their competitive power over the emergency outbreak, in terms of proportion of awarded contracts on the relevant economic market.
#'
#' ### Scoring Rule:
#' The output will give $1 - pvalue$, which will then be dichotomised to 1 if statistical test is significant, 0 otherwise.
#'
#' ### Main target unit:
#' This indicator targets **Companies**
#'
#' @param data This argument should be a data frame or tibble containing the data you want to use to calculate the indicator.
#' @param publication_date This argument corresponds to the name of the column in data containing the publication date for each notice or report.
#' @param emergency_name This argument should be a character string specifying the name of the emergency or event you are analyzing. Examples could include "Coronavirus" or "Terremoto Aquila".
#' @param test_type  This argument should be a character vector specifying the type of hypothesis test to apply to the data. Available options are "barnard", "fisher", or "z-test".
#' @param stat_unit This argument should be a character string specifying the statistical unit of measurement or aggregation variable for the indicator. In this indicator companies are the targets
#' @return indicator schema as from [generate_indicator_schema()]
#' @examples
#' \dontrun{
#' if (interactive()) {
#'   ind_1(
#'     data = mock_data_core,
#'     publication_date = data_pubblicazione,
#'     stat_unit = cf_amministrazione_appaltante,
#'     emergency_name = "coronavirus",
#'     test_type = "fisher"
#'   )
#' }
#' }
#' @seealso
#'  \code{\link[lubridate]{ymd}}
#'  \code{\link[dplyr]{mutate}}, \code{\link[dplyr]{if_else}}, \code{\link[dplyr]{group_by}}, \code{\link[dplyr]{summarise}}, \code{\link[dplyr]{context}}, \code{\link[dplyr]{filter}}, \code{\link[dplyr]{rowwise}}, \code{\link[dplyr]{select}}, \code{\link[dplyr]{reexports}}
#'  \code{\link[stringr]{str_sub}}
#'  \code{\link[tidyr]{nest}}
#' @rdname ind_1
#' @export
#' @importFrom lubridate ymd
#' @importFrom dplyr mutate if_else group_by summarise n filter rowwise select contains
#' @importFrom stringr str_sub
#' @importFrom tidyr unnest
ind_1 <- function(data,
                  publication_date,
                  emergency_name,
                  stat_unit,
                  test_type) {
  indicator_id <- 1
  indicator_name <- "Winning rate across the crisis"
  aggregation_type <- quo_squash(enquo(stat_unit))
  emergency_scenario <- emergency_dates(emergency_name)
  cpvs <- get_associated_cpv_from_emergency(emergency_scenario$em_name)
  cpv_col <- grab_cpv(data = data)

  test <- function(a, b, c, d, test_type) {
    switch(test_type,
      "barnard" = {
        compute_barnard(a, b, c, d)
      },
      "fisher" = {
        compute_fisher(a, b, c, d)
      },
      "z-test" = {
        compute_prop_test(a, b, c, d)
      },
      stop(paste0("No handler for ", test_type))
    )
  }

  data %>%
    dplyr::mutate(
      prepost = dplyr::if_else(lubridate::ymd({{ publication_date }}) >= emergency_scenario$em_date,
        true = "post",
        false = "pre"
      ),
      prepost = factor(prepost, levels = c("post", "pre")),
      flagdivision = dplyr::if_else(stringr::str_sub(.data[[cpv_col]], start = 1, end = 2) %in% cpvs,
        true = 1,
        false = 0
      )
    ) %>%
    dplyr::filter(!is.na({{ stat_unit }})) %>%
    dplyr::group_by({{ stat_unit }}) %>%
    dplyr::summarise(
      n = dplyr::n(),
      n_11 = sum(flagdivision == 0 & prepost == "pre"),
      n_12 = sum(flagdivision == 1 & prepost == "pre"),
      n_21 = sum(flagdivision == 0 & prepost == "post"),
      n_22 = sum(flagdivision == 1 & prepost == "post"),
      m_1 = n_11 + n_12,
      m_2 = n_21 + n_22,
      p_1 = n_12 / m_1,
      p_2 = n_22 / m_2,
      diff_p2_p1 = p_2 - p_1
    ) %>%
    # dplyr::filter(!is.na(p_1) & !is.na(p_2)) %>%
    dplyr::filter(n_12 > 0 | n_22 > 0) %>% # at least one contract in selected CPVs
    dplyr::rowwise() %>%
    dplyr::mutate(
      ## apply test
      tab = paste(n_11, n_12, n_21, n_22, sep = "-"),
      test = test(n_11, n_12, n_21, n_22, test_type)[1],
      # new companies --> at risk
      test = dplyr::if_else(m_1 == 0 & n_22 > 0,
        true = 0,
        false = test
      )
    ) %>%
    dplyr::select({{ stat_unit }}, dplyr::contains("test")) %>%
    generate_indicator_schema(
      indicator_id = indicator_id,
      indicator_name = indicator_name,
      indicator_value = 1 - test, # 1 - pvalue
      aggregation_name = {{ stat_unit }},
      aggregation_type = as_string(aggregation_type),
      emergency = emergency_scenario
    ) %>%
    return()
}
